@@ -11,16 +11,20 @@
 module.exports = {
   meta: {
     docs: {
-      url:
-        "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/no-useless-run-test.html",
+      url: "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/no-useless-run-test.html",
     },
     fixable: "code",
+    messages: {
+      noUselessRunTest:
+        "Useless run_test function - only contains run_next_test; whole function can be removed",
+    },
+    schema: [],
     type: "suggestion",
   },
 
   create(context) {
     return {
-      "Program > FunctionDeclaration": function(node) {
+      "Program > FunctionDeclaration": function (node) {
         if (
           node.id.name === "run_test" &&
           node.body.type === "BlockStatement" &&
@@ -46,7 +50,7 @@ module.exports = {
               // rather than the token before the comments, so that we don't
               // remove the comments - for run_test, these are likely to be useful
               // information about the test.
-              if (startNode && startNode.length) {
+              if (startNode?.length) {
                 startNode = startNode[startNode.length - 1];
               } else {
                 startNode = sourceCode.getTokenBefore(node);
@@ -63,8 +67,7 @@ module.exports = {
                 node.range[1] + 1,
               ]);
             },
-            message:
-              "Useless run_test function - only contains run_next_test; whole function can be removed",
+            messageId: "noUselessRunTest",
           });
         }
       },
