@@ -11,12 +11,12 @@
 module.exports = {
   meta: {
     docs: {
-      url:
-        "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/reject-requires-await.html",
+      url: "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/reject-requires-await.html",
     },
     messages: {
       rejectRequiresAwait: "Assert.rejects needs to be preceded by await.",
     },
+    schema: [],
     type: "problem",
   },
 
