@@ -2,12 +2,12 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import, print_function, unicode_literals
-
 import os
 import webbrowser
 from threading import Timer
 
+from gecko_taskgraph.target_tasks import filter_by_uncommon_try_tasks
+
 from tryselect.cli import BaseTryParser
 from tryselect.push import (
     check_working_directory,
@@ -16,8 +16,6 @@ from tryselect.push import (
 )
 from tryselect.tasks import generate_tasks
 
-from taskgraph.target_tasks import filter_by_uncommon_try_tasks
-
 here = os.path.abspath(os.path.dirname(__file__))
 
 
@@ -48,12 +46,14 @@ def run(
     save=False,
     preset=None,
     mod_presets=False,
-    push=True,
+    stage_changes=False,
+    dry_run=False,
     message="{msg}",
     closed_tree=False,
 ):
     from .app import create_application
 
+    push = not stage_changes and not dry_run
     check_working_directory(push)
 
     tg = generate_tasks(parameters, full)
@@ -91,6 +91,7 @@ def run(
         "chooser",
         message.format(msg=msg),
         try_task_config=generate_try_task_config("chooser", selected, try_config),
-        push=push,
+        stage_changes=stage_changes,
+        dry_run=dry_run,
         closed_tree=closed_tree,
     )
