@@ -291,6 +291,8 @@ class AutoTry:
         "mochitest": ["mochitest-1", "mochitest-e10s-1"],
         "xpcshell": ["xpcshell"],
         "chrome": ["mochitest-o"],
+        "browser-a11y": ["mochitest-ba"],
+        "browser-media": ["mochitest-bmda"],
         "browser-chrome": [
             "mochitest-browser-chrome-1",
             "mochitest-e10s-browser-chrome-1",
@@ -312,6 +314,8 @@ class AutoTry:
         "xpcshell": "xpcshell",
         "chrome": "mochitest-o",
         "browser-chrome": "mochitest-bc",
+        "browser-a11y": "mochitest-ba",
+        "browser-media": "mochitest-bmda",
         "devtools-chrome": "mochitest-dt",
         "crashtest": "crashtest",
         "reftest": "reftest",
@@ -370,6 +374,12 @@ class AutoTry:
                 if "subsuite" in t and t["subsuite"] == "devtools":
                     flavor = "devtools-chrome"
 
+                if "subsuite" in t and t["subsuite"] == "a11y":
+                    flavor = "browser-a11y"
+
+                if "subsuite" in t and t["subsuite"] == "media-bc":
+                    flavor = "browser-media"
+
                 if flavor in ["crashtest", "reftest"]:
                     manifest_relpath = os.path.relpath(t["manifest"], self.topsrcdir)
                     paths_by_flavor[flavor].add(os.path.dirname(manifest_relpath))
@@ -687,7 +697,8 @@ class AutoTry:
         push_to_try(
             "syntax",
             kwargs["message"].format(msg=msg),
-            push=kwargs["push"],
+            stage_changes=kwargs["stage_changes"],
+            dry_run=kwargs["dry_run"],
             closed_tree=kwargs["closed_tree"],
         )
 
